/********************************************************
** Copyright 1999 Earth Resource Mapping Pty Ltd.
** This document contains unpublished source code of
** Earth Resource Mapping Pty Ltd. This notice does
** not indicate any intention to publish the source
** code contained herein.
**
** FILE:   	NCSMalloc.h
** CREATED:	Thu Feb 25 09:19:00 WST 1999
** AUTHOR: 	Simon Cope
** PURPOSE:	NCS Memory Pool types and protos.
** EDITS:
 *******************************************************/

#ifndef NCSMEMPOOL_H
#define NCSMEMPOOL_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NCSDEFS_H
#include "NCSDefs.h"
#endif
#ifndef NCSARRAY_H
#include "NCSArray.h"
#endif
#ifndef NCSMUTEX_H
#include "NCSMutex.h"
#endif
#ifndef NCSTIMESTAMP_H
#include "NCSTimeStamp.h"
#endif
#ifndef NCSTYPES_H
#include "NCSTypes.h"
#endif

/*
** Typedefs
*/

typedef struct {
	INT32 nElementsInUse;
	INT32 iLastFreeElement;
	void *pElements;
	BOOLEAN *pbElementInUse;
} NCSPoolNode;

typedef struct {
	UINT32 iElementSize;
	UINT32 nElementsPerNode;
	UINT32 nNodes;
	UINT32 nElementsInUse;

	UINT32 nAddNodes;
	NCSTimeStampMs tsAddNodeTime;
	
	UINT32 nRemoveNodes;
	NCSTimeStampMs tsRemoveNodeTime;
	
	UINT32 nAllocElements;
	NCSTimeStampMs tsAllocElementTime;
	
	UINT32 nFreeElements;
	NCSTimeStampMs tsFreeElementTime;

	NCSMutexStats msPool;
} NCSPoolStats;

typedef struct {
	NCSMutex		mMutex;
	NCSPoolStats	psStats;
	UINT32			nMaxElements;
	BOOLEAN			bCollectStats;
	NCSPoolNode		*pNodes;
} NCSPool;

/*
** Prototypes.
*/
NCSPool *NCSPoolCreate(UINT32 iElementSize, UINT32 nElements);
void *NCSPoolAlloc(NCSPool *pPool, BOOLEAN bClear);
void NCSPoolFree(NCSPool *pPool, void *pPtr);
void NCSPoolDestroy(NCSPool *pPool);
void NCSPoolSetMaxSize(NCSPool *pPool, UINT32 nMaxElements);
NCSPoolStats NCSPoolGetStats(NCSPool *pPool);
void NCSPoolEnableStats(NCSPool *pPool);
void NCSPoolDisableStats(NCSPool *pPool);

#ifdef __cplusplus
}
#endif

#endif /* NCSMEMPOOL_H */
