/**********************************************************************
 * $Id: OGRSpatialReference.java,v 1.6 2006/10/03 16:58:57 nacho Exp $
 *
 * Name:     OGRSpatialReference.java
 * Project:  JGDAL. Interface java to gdal (Frank Warmerdam).
 * Purpose:   
 * Author:   Nacho Brodin, brodin_ign@gva.es
 *
 **********************************************************************/
/*Copyright (C) 2004  Nacho Brodin <brodin_ign@gva.es>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package es.gva.cit.jogr;


import java.io.*;
import java.util.Date;

//import es.gva.cit.wrappergdal.OGRException;

/** 
 * Representa un sistema de referencia espacial OpenGIS.
 * 
 * @author Nacho Brodin <brodin_ign@gva.es>.<BR> Equipo de desarrollo gvSIG.<BR> http://www.gvsig.gva.es
 * @version 0.0
 * @link http://www.gvsig.gva.es
 */

public class OGRSpatialReference extends JNIBase{
	
	//private native int setUTMNat(long cPtr, int zona, int norte_sur);
	//private native int setWellKnownGeogCSNat(long cPtr, String cs);
	//private native long OGRSpatialReferenceNat();
	//private native String exportToWktNat(long cPtr);
	private native void OSRDestroySpatialReferenceNat(long cPtr);
	private native int getUTMZoneNat(long cPtr);
	private native int getHemisphereNat(long cPtr);
	private native int setFromUserInputNat(long cPtr, String a_srs);
	
	private native long OGRSpatialReferenceNat() throws CrsGdalException;
	private native void OGRDestroySpatialReferenceNat(long crs);
	
	protected static native String exportToProj4Nat(long cPtr);
	protected static native String exportToWktNat(long cPtr);
	protected static native int importFromWktNat(long cPtr, String cadenas);
	protected static native int setUTMNat(long cPtr, int zona, int norte_sur);
	protected static native int setWellKnownGeogCSNat(long cPtr, String cs);
	protected static native int importFromEPSGNat(long cPtr, int cod);
	protected static native int importFromProj4Nat(long cPtr, String cs);
	protected static native int importFromPCINat(long cPtr,String cod, String cs,double[] coord);
	protected static native int importFromUSGSNat(long cPtr, long code, long zone, double[] params, long datum);
	protected static native int importFromESRINat(long cPtr, String cadenas);
	
	/**
	 *Constructor a partir de la direccin de memoria 
	 */
	
	public OGRSpatialReference(long cPtr){
		this.cPtr=cPtr;
	}
	
	/**
	 * Constructor generico 
	 */
	
	public OGRSpatialReference(){
		try{
			cPtr = OGRSpatialReferenceNat();
			if(cPtr <= 0) 
				throw new CrsGdalException();
		}catch(CrsGdalException exp){
			System.out.println(exp.getStrError());
		}
	}
	
	/**
	 * Escribe la zona UTM pasada por parmetro
	 * @param zona	Zona UTM
	 * @param norte_sur	TRUE hemisferio norte y FALSE hemisferio sur
	 * @throws OGRException
	 */
	
	public void setUTM(int zona, boolean norte_sur)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setUTM(). La llamada de creacin de objeto no tuvo exito");
		
		int ns=-1;
		if(norte_sur)ns=1;
		else ns=0;
		
		int res = setUTMNat(cPtr, zona, ns);
		
		if(res<0)
			throw new OGRException("Error en setUTM(). No se ha podido asignar la zona especificada.");
	}
	
	/**
	 * Lee la zona UTM 
	 * @return Zona UTM
	 * @throws OGRException
	 */
	
	public int getUTMZone()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en getUTMZone(). La llamada de creacin de objeto no tuvo exito");
		
		int res = getUTMZoneNat(cPtr);
		
		if(res<0)
			throw new OGRException("Error en getUTMZone(). Valor de zona devuelto erroneo.");
		
		return res;	
	}
	
	/**
	 * Obtiene el hemisferio
	 * @return TRUE hemisferio norte y FALSE hemisferio sur
	 * @throws OGRException
	 */
	
	public boolean getHemisphere()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en getHemisphere(). La llamada de creacin de objeto no tuvo exito");
		
		int hemis = getHemisphereNat(cPtr);
		 			
		if(hemis<0)
			throw new OGRException("Error en getHemisphere(). Valor de hemisferio devuelto erroneo.");
		
		if(hemis==1)return true;
		else return false;
	}
	
	/**
	 *Asigna las coordenadas geogrficas
	 *@param cs	Coordenadas geograficas soportadas:<BR>
	 *<UL>
	 *<LI>WGS84</LI>
	 *<LI>WGS72</LI>
	 *<LI>NAD27</LI>
	 *<LI>NAD83</LI>
	 *<LI>EPSG:n</LI>
	 *</UL>
	 *@throws OGRException
	 */
	
	public void setWellKnownGeogCS( String cs)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setWellKnownGeogCS(). La llamada de creacin de objeto no tuvo exito");
		
		int res = setWellKnownGeogCSNat(cPtr, cs);
		
		if(res<0)
			throw new OGRException("Error en setWellKnownGeoCS(). No se ha podido asignar el sistema de coordenadas especificado.");
	}
	
	/**
	 * Devuelve la codificacin WKT
	 * @return Codificacin WKT
	 * @throws OGRException
	 */
	public String exportToWkt()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en exportToWkt(). La llamada de creacin de objeto no tuvo exito");
		
		String wkt = exportToWktNat(cPtr);
		
		if(wkt==null)
			throw new OGRException("Error en exportToWkt(). No se ha podido obtener la proyeccin.");
		
		return wkt;
	}
	
	/**
	 * @throws OGRException
	 * @param a_srs	
	 */
	public void setFromUserInput(String a_srs)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setFromUserInput(). La llamada de creacin de objeto no tuvo exito");
		
		int ogrerr = setFromUserInputNat(cPtr,a_srs);
		lanzarExcepcion(ogrerr,"Error en setFromUserInput().");
	}
	
	/**
	 * Destructor 
	 */
	protected void finalize(){
		if(cPtr > 0)
			OSRDestroySpatialReferenceNat(cPtr);
	}
	
	
	//****************************************
	//Llamadas de Miguel Garca
	//Estas deberan funcionar al integrar su jar en jgdal. Al hacer esto deber cambiarse la cabecera
	//de las llamadas de C ya que el nombre del paquete corresponde con el de su proyecto y no con este.
	
	/*public static String exportToWkt(OGRSpatialReference ORGSpace)throws CrsOgrException, crsgdalException
	{
		String result = exportToWktNat(ORGSpace.getPtr());
		if(result.length() == 0) throw new CrsOgrException(ORGSpace);
		return result;
	}
	
	public int importFromWkt(OGRSpatialReference ORGSpace, String cadenas) throws CrsGdalException, CrsOgrException, OGRException{
		int result = importFromWktNat(this.cPtr, cadenas);
		if(result != 0) throw new OGRException(result,"Error en Wkt().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public int setUTM(OGRSpatialReference ORGSpace ,int zona, boolean norte_sur)throws OGRException, CrsGdalException, CrsOgrException{	
		if(this.cPtr <= 0)
			throw new OGRException("Error en setUTM(). La llamada de creacin de objeto no tuvo exito.");
		
		int ns=-1;
		if(norte_sur)
			ns=1;
		else 
			ns=0;
		
		int res = setUTMNat(this.cPtr, zona, ns);
		
		if(res != 0)
			throw new OGRException(res,"Error en setUTM(). No se ha podido asignar la zona especificada. ",ORGSpace);
		return res;
	}
	
	public int setWellKnownGeogCS(OGRSpatialReference ORGSpace, String cs)throws OGRException, CrsGdalException, CrsOgrException{
		if(this.cPtr <= 0)
			throw new OGRException("Error en setWellKnownGeogCS(). La llamada de creacin de objeto no tuvo exito.");
		
		int res = setWellKnownGeogCSNat(this.cPtr, cs);
		
		if(res != 0)
			throw new OGRException(res,"Error en setWellKnownGeoCS(). No se ha podido asignar el sistema de coordenadas especificado. ",ORGSpace);
		return res;
	}
	
	public int importFromEPSG(OGRSpatialReference ORGSpace, int cod) throws OGRException, CrsGdalException, CrsOgrException{
		int result = importFromEPSGNat(this.cPtr, cod);
		if(result != 0) 
			throw new OGRException(result,"Error en EPSG().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public int importFromProj4(OGRSpatialReference ORGSpace, String cs)throws OGRException, CrsGdalException, CrsOgrException{
		if(this.cPtr <= 0)
			throw new OGRException("Error en Proj4(). La llamada de creacin de objeto no tuvo exito.");
		
		int res = importFromProj4Nat(this.cPtr, cs);
		
		if(res != 0)
			throw new OGRException(res, "Error en Proj4(). No se ha podido asignar el sistema de coordenadas especificado. ", ORGSpace);
		
		return res;
		
	}
	
	public String exportToProj4(OGRSpatialReference ORGSpace)throws CrsOgrException, CrsGdalException{
		String result = exportToProj4Nat(this.cPtr);
		if(result.length() == 0) 
			throw new CrsOgrException();
		return result;
	}
	
	public int importFromPCI(OGRSpatialReference ORGSpace,String cod, String cs, double[] coord)throws CrsOgrException, CrsGdalException, OGRException{
		int result = importFromPCINat(this.cPtr, cod, cs, coord);
		if(result != 0) 
			throw new OGRException(result,"Error en PCI().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public int importFromUSGS(OGRSpatialReference ORGSpace, long code, long zone, double[] params, long datum) throws CrsGdalException, OGRException, CrsOgrException{
		int result = importFromUSGSNat(this.cPtr, code, zone, params, datum);
		if(result != 0) 
			throw new OGRException(result,"Error en USGS().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;	
		
	}
	
	public int importFromESRI(OGRSpatialReference ORGSpace, String cadenas) throws CrsGdalException, CrsOgrException, OGRException{
		int result = importFromESRINat(this.cPtr, cadenas);
		if(result != 0) 
			throw new OGRException(result,"Error en ESRI().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}*/
}